;******************************************************************************
; Dateiname:	Intervallschalter mit 12f675
; Autor:		Andreas Heger
; Datum:  		Januar 2010
; Version:		1.00
;
; Letzte nderung:	keine 
;			
;******************************************************************************
; Funktionsbeschreibung des Programms:
;
; Einstelbare Intervallzeit, verzgertes Wischer ansprechen bei Wisch-Wasch, Regensensoreingang
; Hauptprogramm Unterbrechung bei Wisch-Wasch Betrieb
;
;******************************************************************************
	list      	p=12F675			
	#include 	<p12f675.inc>		

	__CONFIG  _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & _CPD_OFF
;
;
;				CP: Code Protection bit
;              *		OFF = Program Memory code protection is disabled 
;                  		ON  = Program Memory code protection is enabled
;              	WDT: Watchdog Timer Enable bit
;                  		ON  = WDT enabled
;               *  		OFF = WDT disabled 
;               BODEN: Brown-out Detect Enable bit
;               * 	 	ON  = BOD enabled 
;                   	OFF = BOD disabled
;               PWRTE: Power-up Timer Enable bit
;                     	OFF = PWRT disabled
;              	*		ON  = PWRT enabled
;               MCLRE: GP3/MCLR pin function select
;                      	ON  = GP3/MCLR pin function is MCLR
;               *   	OFF = GP3/MCLR pin function is digital I/O
; 				CPD: Data Code Protection bit 
;               *   	OFF = Data memory code protection is disabled 
;                      	ON  = Data memory code protection is enabled
;				Oscillator Variations:
;						LP_OSC   = Low power crystal on GPIO4 and GPIO5 
;						XT_OSC   = Crystal/resonator on GPIO4 and GPIO5 
;						HS_OSC   = High speed crystal/resonator on GPIO4 and GPIO5 
;						EC_OSC   = I/O function on GPIO4 pin, CLKIN on GPIO5 
;					*	INTRC_OSC_NOCLKOUT  = I/O function on GPIO4 pin, I/O function on GPIO5 
;						INTRC_OSC_CLKOUT  = CLKOUT function on GPIO4 pin, I/O function on GPIO5 
;						EXTRC_OSC_NOCLKOUT  = I/O function on GPIO4 pin, RC on GPIO5 
;						EXTRC_OSC_CLKOUT    = CLKOUT function on GPIO4 pin, RC on GPIO5
;   
;
	ERRORLEVEL      -302    	; Unterdrcken BANK SELECTION MESSAGES
;								; Dies unterdrckt im Compiler die Nachricht "Register in operand not in bank 0.  
;                               ; Ensure that bank bits are correct"	
;
;*****************************************************************************
; Deklaration der Variablen und Konstanten
;******************************************************************************
;	
schleife	EQU	H'0020'							; Anzahl der "schleifen"
help		EQU	H'0021'							; Hilfsvariable 
help1       EQU H'0022'                         ; Hilfsvariable  
sekunde2    EQU H'0023'                         ; Hilfsvariable 
sekunde4    EQU H'0024'                         ; Hilfsvariable 
sekunde6    EQU H'0025'                         ; Hilfsvariable 
sekunde8    EQU H'0026'                         ; Hilfsvariable 
potizeit    EQU H'0027'                         ; Hilfsvariable Intervallzeit
w_copy		EQU H'0028'							; Variable zum Retten des W-Registers 
s_copy		EQU	H'0029'							; Variable zum Retten des Status-Registers 
;
;*****************************************************************************
; Startadresse des PIC-Controllers nach einem RESET oder Neustart
;*****************************************************************************

		org		0x000							; Startadresse des PICs 
		goto	START
		
;*****************************************************************************
; Sprungadresse bei einem Interrupt (ISR Interupt Service Routine)                      
;*****************************************************************************
;
		org 	0X004 							; Startadresse der ISR
		

		bcf		STATUS,RP0						; Sicherstellen, dass der Controller in Bank 0 is

wiwa                                            ;durch das Interrupt ist das Wisch-Waschen immer vorrangig 
        movlw   D'249'                          
        movwf   help

        movlw   D'8'
        movwf   sekunde2

        movlw   D'16'
        movwf   sekunde4

        movlw   D'24'
        movwf   sekunde6

        movlw   D'32'
        movwf   sekunde8

        bcf     GPIO,2
        btfss    GPIO,3
        goto     wiwa

        
        
        call    PAUSE2s                         ; warte bis die Scheibe nass ist mit dem Wischerstart
        bsf     GPIO,2
        call    PAUSE6s                         ; Wischenzeit
        bcf     GPIO,2
        call    PAUSE8s
        bsf     GPIO,2
        call    PAUSE4s                         ; Nachwischzeit 
        bcf     GPIO,2
       

fertig											; Beenden der IRS
		banksel INTCON
        bcf		INTCON,GPIE                     ; Interrupt-Flag lschen
					

        goto START                              ; Rcksprung ins Hauptprogramm
								
			
;******************************************************************************
; PIC Einstellungen
;******************************************************************************
;
START	
		bsf     STATUS,RP0						; Auswahl Bank 1 
		call 	0x3FF							; Laden des Kalibrierungswertes aus der letzten Speicherstelle 
		movwf   OSCCAL							; Schreiben des Kalibrierungswertes fr den internen Takt
		
		movlw	B'00010001'						; FOCS 8 und AN0
		movwf	ANSEL
	 											
		movlw	B'00111001'      			    ; Pins konfigurieren
		movwf	TRISIO							; Schreiben der Werte
		bcf     STATUS,RP0						; Zurck zu Bank 0 
		
; Interrupt
        movlw   B'00000000'                     ; EEIE | ADIE | x | x | CMIE | x | x | TMR1E disable interrupts
        banksel PIE1
        movwf   PIE1

        movlw   B'00001000'                     ; enable IOC of Port3 (Switch, Flanke an Port 3 lst Interuppt aus)
        banksel IOC
        movwf   IOC

        movlw   B'00001000'                     ;| GPIE | Interuppt wird extern durch Port ausgelst
        banksel INTCON                          ; enable only port change interrupt banksel INTCON
        movwf   INTCON
        banksel INTCON
       
        bsf     INTCON, GIE                     ; enable global interrupts (Interrupt grundstzlich einschalten)

		
	   	movlw   B'00000001'     			   	; Linksbndig, AN 0, AD an
		movwf   ADCON0
		
		clrf	GPIO					        ; Lschen aller Ausgnge 
	
;******************************************************************************
; Hauptprogramm
;******************************************************************************
;
HAUPT                                             ; Beginn des Hauptprogramms
                          
        movlw   D'249'                          
        movwf   help

        movlw   D'8'
        movwf   sekunde2

        movlw   D'16'
        movwf   sekunde4

        movlw   D'24'
        movwf   sekunde6

        movlw   D'32'
        movwf   sekunde8


										    	
		btfsc 	GPIO,4							; Abfragen des Eingangs GPIO4, ob Taster gedrckt wird
		goto	regen							; Taste nicht gedrckt, springe zur Marke "regen"	

BLINK
		bsf    	ADCON0,GO    					; A/D-Wandler starten
AD_W
       	btfsc  	ADCON0,GO    					; Ist der A/D-Wandler fertig?
       	goto   	AD_W          					; Nein, weiter warten
	
		bsf 	GPIO,2        
							
INTERV	call	PAUSE2s					     	; Aufruf des Unterprogramms "Pause2s"
		bcf     GPIO,2					      	; lsche Ausgang

		movfw	ADRESH							; Dezimalwert fr die Zeitkonstante zur bergabe				
		addlw	D'1'							; Addition der Mindest-Verzgerung
		movwf	potizeit						; Schieben des Wertes in die Variable "potizeit" (Intervallzeit)
		call	Zeit 
        goto    HAUPT 
 

                                  
regen   btfsc   GPIO,5                          ;Abfrage des Eingang GPIO,5                       
     	goto	HAUPT
        call    BLINK
        goto    HAUPT

;******************************************************************************
; Unterprogramme
;******************************************************************************
;******************************************************************************
; Standardwarteschleife fr 1 ms bei 4MHz Systemtakt
;
PAUSE1ms					
		movlw	D'199'		
		movwf	schleife	
							
PAUSEE						
		nop					
		nop
   
		decfsz	schleife,1	
		goto	PAUSEE		
							 
        return
;********************************************************************************


PAUSE250ms				
		call	PAUSE1ms
		decfsz 	help,1		
		goto	PAUSE250ms					
			
		return
;*********************************************************************************

PAUSE2s				      	
		call	PAUSE250ms
		decfsz 	sekunde2,1	
		goto	PAUSE2s    					
			
		return
;********************************************************************************

PAUSE4s				      	
		call	PAUSE250ms
		decfsz 	sekunde4,1	
		goto	PAUSE4s    					
			
		return
;*********************************************************************************

PAUSE6s				      	
		call	PAUSE250ms
		decfsz 	sekunde6,1	
		goto	PAUSE6s    					
			
		return
;*********************************************************************************

PAUSE8s				      	
		call	PAUSE250ms
		decfsz 	sekunde8,1	
		goto	PAUSE8s    					
			
		return
;********************************************************************************
; Intervallzeit

Zeit 		
		call	PAUSE250ms
		decfsz 	potizeit,1	
		goto	Zeit    					
			
		return
	
		end


